#ifndef COLORTABLEITEM_H
#define COLORTABLEITEM_H

#include "qtable.h"
#include "qpainter.h"
#include "qpalette.h"
#include "qrect.h"
#include <iostream>

using namespace std;

/*! \enum BREAK_TYPE

    \brief Enum for the type of break on a GUI register
    
    This indicates whether a GUI register has a read, write or no breakpoint
*/
typedef enum _BREAK_TYPE
{
    NO_BREAK,
    READ_BREAK,
    WRITE_BREAK,
} BREAK_TYPE;

/*! \class ColorTableItem

    \brief ColorTableItems are inserted into QTable to make coloured cells
    
    This is a subclass of QTableItem and overrides the paint method to colour
    the cells according to it's BREAK_TYPE
*/
class ColorTableItem : public QTableItem
{
    private:
	BREAK_TYPE breakType; //! The BREAK_TYPE of ColorTableItem
    public:
        //! ColorTableItem constructor
        /*! 
            Creates a new ColorTableItem object
        */
        ColorTableItem( QTable *table, EditType et, const QString& text, BREAK_TYPE bt);
        
        //! The overriden paint function
        /*! 
            This function determines the BREAK_TYPE of the register and repaints
            the ColorTableItem accordingly. White = No Break, Yellow = Read Break
            and Red = Write Break.
            \param p Pointer to a QPainter object
            \param cg Reference to the QColorGroup
            \param cr Reference to the QRect
            \param selected Indicates whether the ColorTableItem is selected
            
        */
        void paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bool selected );
        
        //! Sets the BREAK_TYPE of GUI register
        /*!
            \param t The BREAK_TYPE to set to
        */
	void setType(BREAK_TYPE t);
	
};

#endif

