
#ifndef __GPSIM_CLASSES_H__
#define __GPSIM_CLASSES_H__



/*
 * Define all of the different types of reset conditions:
 */

//! \define Maximum program memory size
#define MAX_PROGRAM_MEMORY 0xffff

//! \define Rate at which GUI updates
#define DEFAULT_GUI_UPDATE_RATE   0x100000

//! \define Maximum HLL files
#define MAX_HLL_FILES 20

//! \enum 
/*!
    \brief The various reset types
*/
enum RESET_TYPE
{
  //! Power on reset
  POR_RESET, 
  //! Watch Dog Timer reset
  WDT_RESET, 
  //! Soft reset
  SOFT_RESET 
};


//! \enum 
/*!
    \brief The various simulation modes
*/
enum SIMULATION_MODES
{
  //! Stopped
  STOPPED,   
  //! Abnormal stop        
  ABNORMAL_STOP,  
  //! Executing   
  RUNNING,      
  //! Sleeping    
  SLEEPING,    
  //! Single stepping     
  SINGLE_STEPPING, 
  //! Stepping over 
  STEPPING_OVER,   
  //! Running over 
  RUNNING_OVER, 
  //! PM write   
  PM_WRITE          
};


//! \enum 
/*!
    \brief The various processor states
*/
enum PROCESSOR_STATES
{
  //! Power On Reset state
  POR_,  
  //! Idle state   
  IDLE      
};

#endif //  __GPSIM_CLASSES_H__
