
#include "processor.h"
#include "hardware_mapper.h"

#include <unistd.h>
#include <sys/io.h>
#include <math.h>

#define LPT_0 0x278
#define LPT_1 0x378
#define LPT_2 0x3BC 

#define DATA_OFFSET 0
#define STAT_OFFSET 1
#define CTRL_OFFSET 2

//Constructor
HardwareMapper::HardwareMapper(Processor *p) 
{
   cpu = p;
   //Need to change this if some pins are active_low or start high, etc
   for(int i=0; i<PIC_PINS; i++)
   {
      picPinMap[i] = -1;
      currPinValue[i] = 0;
      prevPinValue[i] = 0;
      //pinIOStatus[i] = 0;
   }
   unsigned int trisa = cpu->trisa.get();
   unsigned int trisb = cpu->trisb.get();
   
   pinIOStatus[0] = (trisa >> 2)& 0x01;
   pinIOStatus[1] = (trisa >> 3)& 0x01;
   pinIOStatus[2] = (trisa >> 4)& 0x01;
   pinIOStatus[3] = 2;
   pinIOStatus[4] = 2;
   pinIOStatus[5] = trisb & 0x01;
   pinIOStatus[6] = (trisb >> 1) & 0x01;
   pinIOStatus[7] = (trisb >> 2) & 0x01;
   pinIOStatus[8] = (trisb >> 3) & 0x01;
   pinIOStatus[9] = (trisb >> 4) & 0x01;
   pinIOStatus[10] = (trisb >> 5) & 0x01;
   pinIOStatus[11] = (trisb >> 6) & 0x01;
   pinIOStatus[12] = (trisb >> 7) & 0x01;
   pinIOStatus[13] = 2;
   pinIOStatus[14] = 2;
   pinIOStatus[15] = 2;
   pinIOStatus[16] = trisa & 0x01;
   pinIOStatus[17] = (trisa >> 1) & 0x01;
   
   ports[0] = 12;
   ports[1] = 13;
   ports[2] = 14;
   ports[3] = 0;
   ports[4] = 0;
   ports[5] = 20;
   ports[6] = 21;
   ports[7] = 22;
   ports[8] = 23;
   ports[9] = 24;
   ports[10] = 25;
   ports[11] = 26;
   ports[12] = 27;
   ports[13] = 0;
   ports[14] = 0;
   ports[15] = 0;
   ports[16] = 10;
   ports[17] = 11;
   

   for(int i=0; i<LPT_PINS; i++)
   {
      portPinMap[i] = -1;
   }

   writeMode = 0;
   ioperm(LPT_0, 3, 1);
   ioperm(LPT_1, 3, 1);
   ioperm(LPT_2, 3, 1);
   lpt_add = LPT_1;
}


HardwareMapper::~HardwareMapper()
{
   ioperm(LPT_0, 3, 0);
   ioperm(LPT_1, 3, 0);
   ioperm(LPT_2, 3, 0);
  
}


void HardwareMapper::setLptData(int buttonID)
{
    if(buttonID == 0)
        lpt_add = LPT_0;
    else if(buttonID == 1)
        lpt_add = LPT_1;
    else if(buttonID == 2)
        lpt_add = LPT_2;
}

//The contents of the array parameter dictate whether the indexed pin
//is an input or an output. This information is stored to a local array
//Input is indicated by a 1, output is indicated by a 0
//RETURN VAL
//Success is indicated by '-1'. 
//If the array has invalid data, the index of that data will be returned
//If the combination can not be handled by the simulator, returns -2

int HardwareMapper::checkIOValidity()
{
   int num_in=0, num_out=0, error=0, oldWrite = writeMode;

   if(writeMode == 0)
   {
      return error;
   }
   unsigned int trisa = cpu->trisa.get();
   
   unsigned int trisb = cpu->trisb.get();
   pinIOStatus[0] = (trisa >> 2)& 0x01;
   pinIOStatus[1] = (trisa >> 3)& 0x01;
   pinIOStatus[2] = (trisa >> 4)& 0x01;
   //pinIOStatus[3] = trisa & 0x04;
   //pinIOStatus[4] = trisa & 0x04;
   pinIOStatus[5] = trisb & 0x01;
   pinIOStatus[6] = (trisb >> 1) & 0x01;
   pinIOStatus[7] = (trisb >> 2) & 0x01;
   pinIOStatus[8] = (trisb >> 3) & 0x01;
   pinIOStatus[9] = (trisb >> 4) & 0x01;
   pinIOStatus[10] = (trisb >> 5) & 0x01;
   pinIOStatus[11] = (trisb >> 6) & 0x01;
   pinIOStatus[12] = (trisb >> 7) & 0x01;
   //pinIOStatus[13] = trisb & 0x20;
   //pinIOStatus[14] = trisb & 0x20;
   //pinIOStatus[15] = trisb & 0x20;
   pinIOStatus[16] = trisa & 0x01;
   pinIOStatus[17] = (trisa >> 1) & 0x01;
   
   
   for(int index=0; index<PIC_PINS; index++)
   {
      if(pinIOStatus[index] == 0)
         num_out++;
      else if(pinIOStatus[index] == 1)
         num_in++;
      else if(pinIOStatus[index] == 2)
         continue;
      else
      {
         error = index;
         break;
      }
   }

   if(error)
      return error;
   else if(num_in >= 0 && num_in <= 5 && num_out >= 0 && num_out <= 12)
   {
      writeMode = 2;

      /* Data pins are out */
      outb(inb(lpt_add+CTRL_OFFSET) & 0xDF, lpt_add+CTRL_OFFSET);      
   
      if(oldWrite!=writeMode)
      {
         return -2;
      }
      return -1;
   }
   else if(num_in >= 0 && num_in <= 13 && num_out >= 0 && num_out <= 4)
   {
      writeMode = 1;
      
      /* Data pins are in */
      outb(inb(lpt_add+CTRL_OFFSET) | 0x20, lpt_add+CTRL_OFFSET);
   
      if(oldWrite!=writeMode)
      {
         return -2;
      }
      return -1;
   }
   else 
      return -2;
}


/*int HardwareMapper::setIOPinDirection(const int PICpin, const int value)
{
   pinIOStatus[PICpin] = value;
   return checkIOValidity();
}*/




//INPUT mode only 1 or 2
void HardwareMapper::setIOMode(const int mode)
{
   writeMode = mode;
   
   if(writeMode == 2)
      outb(inb(lpt_add+CTRL_OFFSET) & 0xDF, lpt_add+CTRL_OFFSET);      
   else if(writeMode == 1)
      outb(inb(lpt_add+CTRL_OFFSET) | 0x20, lpt_add+CTRL_OFFSET);   
}

//Implementation of these read functions are dependant upon which format
//the user has selected for the I/O combinations on the PIC.
//NB: Expected Range: 1 <= picPin <= 18
int HardwareMapper::readSingle(const int picPin)
{
   //Erroneous pin request
   if(picPin == 0)
      return -1;
   
   return currPinValue[picPin - 1];
}


int* HardwareMapper::readArray()
{
   int* tempArray = new int[PIC_PINS];

   for(int i=0; i<PIC_PINS; i++)
   {
      //tempArray[i] = 
   }

   return tempArray;
}

//Expected Range: 1 <= picPins[x] <= 18
int* HardwareMapper::readArray(const int picPins[], const int numPins)
{
   int* tempArray = new int[numPins];
 
   for(int i=0; i<numPins; i++)
   {
      tempArray[i] = readSingle(picPins[i]);
   }

   return tempArray;
}

//An array representing all of the pins of the PIC is passed in
//Pins can either be set High(1), Low(0), or unchanged(-1)
//All previous values, regardless of change, are shunted to prevPinValue
//Returns true if the operation was successful.
//CHECK IF R/W PIN
bool HardwareMapper::setPins(const int picPins[])
{
   for(int i=0; i<PIC_PINS; i++)
   {
      prevPinValue[i] = currPinValue[i];
      
      if(picPins[i] != -1)
         currPinValue[i] = picPins[i];
   } 

   return true;
}

bool HardwareMapper::write()
{
   
   char old_ctrl;
   char byte1=0x00;
   char byte2=0x00;
   unsigned int shifter = 0x00;
   int temp;
   /*0-13 inputs, 0-4 outputs*/
   int control_lpt[] = {17,16,14,1};
   int data_lpt[] = {9,8,7,6,5,4,3,2};
   
   old_ctrl = inb(lpt_add+CTRL_OFFSET);
   old_ctrl &= 0xF0;
   
   oldPortA = cpu->porta->value;
   oldPortB = cpu->portb->value;
   
   //printPinMap();
   for(int i=0; i<=3; i++)
   {
      temp = ports[getPicPin(control_lpt[i])-1];
      
      if(temp>=20)
      {
         shifter = cpu->portb->get_bit(temp%20);
      }
      else if(temp>=10)
      {
         shifter = cpu->porta->get_bit(temp%10);
      }
      
      shifter = shifter << (3-i);
      
      byte1 |= shifter;
      shifter = 0x00;
   }
   
   byte1 ^= 0x0B;
   byte1 &= 0x0F;
   byte1 |= old_ctrl;
   
   
      
   if(writeMode==2)
   {
      for(int i=0; i<=7; i++)
      {
         temp = ports[getPicPin(data_lpt[i])-1];
  
         if(temp>=20)
         {
            shifter = cpu->portb->get_bit(temp%20);
         }
         else if(temp>=10)
         {
            shifter = cpu->porta->get_bit(temp%10);
         }
         shifter = shifter << (7-i);
         byte2 |= shifter;
         shifter = 0x00;
      }
   }

   outb(byte1,lpt_add+CTRL_OFFSET);
   if(writeMode==2)
      outb(byte2,lpt_add+DATA_OFFSET);
   
   /*for(int i=0; i<PIC_PINS; i++)
   {
      if(pinIOStatus[i] == 0)
      {
         if(picPinMap[i]!=-1)
         {
            unsigned int porta = cpu->porta->get();
            unsigned int portb = cpu->portb->get();
         }
      }
   }*/
   return true;
}

bool HardwareMapper::read()
{
   char byte1=0x00;
   char byte2=0x00;
   char byte3;
   
   oldPortA = cpu->porta->value;
   oldPortB = cpu->portb->value;
   
   byte1 = inb(lpt_add+STAT_OFFSET);

   int status_lpt[] = {0,0,0,15,13,12,10,11};
   int data_lpt[] = {2,3,4,5,6,7,8,9};
   
   for(int i=7; i>2; i--)
   {
      int picPin = getPicPin(status_lpt[i]);
      
      if(picPin!=-1)
      {
         int temp = ports[picPin-1];
         int value = (byte1 & (int)pow(2,i))>>i;
      
      
         //invert 11
         if(status_lpt[i] == 11)
            value = ~value;
         if(temp>=20)
         {
            cpu->portb->setbit(temp%20, (bool)(value&0x01));
         }
         else if(temp>=10)
         {
            cpu->porta->setbit(temp%10, (bool)(value&0x01));
         }
      }
   }
   
   
   if(writeMode == 1)
   {
      byte2 = inb(lpt_add+DATA_OFFSET);
      
      for(int i=7; i>=0; i--)
      {
         int picPin = getPicPin(data_lpt[i]);
         
         if(picPin!=-1)
         {  
            int temp = ports[picPin-1];
            int value = (byte2 & (int)pow(2,i))>>i;
            
            if(temp>=20)
            {
               cpu->portb->setbit(temp%20, (bool)(value&0x01));
            }
            else if(temp>=10)
            {
               cpu->porta->setbit(temp%10, (bool)(value&0x01));
            }
         }
      }
   }
   
   return true;
}

//Returns the direction (0=out/1=in) of the given picPin
int HardwareMapper::getIOStatus(const int picPin)
{
    return pinIOStatus[picPin];
}

//Looks up which port pin is currently being used to interface the
//specified PIC pin. 
//Returns '-1' in the PIC pin is currently unassigned
int HardwareMapper::getPortPin(const int picPin)
{
   return picPinMap[picPin-1];
}


//Looks up which PIC pin is currently being controlled by the specified
//port pin.
//Returns '-1' if the specified port pin is currently unused
int HardwareMapper::getPicPin(const int portPin)
{
   return portPinMap[portPin-1];
}


//Need to set any previous mappings to -1 before setting new values
bool HardwareMapper::mapPin(const int pic, const int port)
{
   if(picPinMap[pic - 1] != -1)
   {
      portPinMap[picPinMap[pic - 1] - 1] = -1;
   }
   
   //Removes the current assignment for PIC pin
   if(portPinMap[port - 1] != -1)
   {
      picPinMap[portPinMap[port - 1] - 1] = -1;
   }
   
   //New assignments
   picPinMap[pic - 1] = port;
   portPinMap[port - 1] = pic;

   return true;
}


bool HardwareMapper::saveMap(const string filename)
{
  return true; 
}


bool HardwareMapper::loadMap(const string filename)
{
   return true;
}


//TESTING FUNCTIONS
void HardwareMapper::printPinMap()
{
   cout << "Port Pins: " << endl;
   for(int i=0; i<LPT_PINS; i++)
   {
      cout << i+1 << ". ";
      if(portPinMap[i] != -1)
      {
         cout << "PIC " << portPinMap[i] << endl;
      }
      else
      {
         cout << "UNASSIGNED" << endl;
      }
   }

   cout << "PIC Pins: " << endl;
   for(int i=0; i<PIC_PINS; i++)
   {
      cout << i+1 << ". ";
      if(picPinMap[i] != -1)
      {
         cout << "LPT " << picPinMap[i] << endl;
      }
      else
      {
         cout << "UNASSIGNED" << endl;
      }
   }

   cout << "Done" << endl;
}

void HardwareMapper::printIOStatus()
{
   cout << "PIC Pins Direction: " << endl;
   for(int i=0; i<PIC_PINS; i++)
   {  
      cout << dec <<"Pin IO Status ["<<i+1<<"]"<< pinIOStatus[i] << endl;  
   }
   cout << "TRISA: " << pinIOStatus[2] << pinIOStatus[1] 
        << pinIOStatus[0] << pinIOStatus[17] << pinIOStatus[16] << endl;
   cout << "TRISB: ";
   for(int i=12; i>4; i--)
   {  
      cout << pinIOStatus[i];
   }
   cout<<endl;
}

int HardwareMapper::getWriteMode()
{
   /*string str = "";
   
   if(writeMode == 0)
      str = "Write mode has not been set\n";
   else if(writeMode == 1)
   {
      str = "Write mode is 1\n";
      str += "0-13 inputs and 0-4 outputs\n";
      str += "Data Byte: Input\n";
      str += "Status Byte: Input\n";
      str += "Control Byte: Output\n\n";
   }
   else if(writeMode == 2)
   {
      str = "Write mode is 2\n";
      str += "0-5 inputs and 0-12 outputs\n";
      str += "Data Byte: Output\n";
      str += "Status Byte: Input\n";
      str += "Control Byte: Output\n\n";
   }
   else
      str = "Error with write mode";

   cout << str;*/
   return writeMode;
}


//Private Function
//Determines if the specified has switched from low to high
bool HardwareMapper::risingEdge(const int picPin)
{
   return currPinValue[picPin - 1] > prevPinValue[picPin - 1];
}


//Private Function
//Determines if the specified has switched from high to low
bool HardwareMapper::fallingEdge(const int picPin)
{
   return currPinValue[picPin - 1] < prevPinValue[picPin - 1];
}
