#include "instruction.h"
#include <iostream>
#include "symbol.h"

Instruction::Instruction() {
  is_modified = 0;
  //cycle_count = 0;
  hll_file_id = 0;
  hll_src_line = -1;
}

Instruction::Instruction(Processor *proc,unsigned int opc,char* nameStr) 
{
   cpu = proc;
   opcode = opc;
   strncpy(name_str,nameStr,sizeof(name_str));
}

void Invalid_Instruction::execute(void)  
{ 
  /* Don't know what to do, so just plow thorugh like nothing happened */
  if(cpu)
    cpu->pc->increment();
};

void Instruction::add_line_number_symbol(int address)
{

  symbol_table.add_line_number(cpu, address);

}

void Instruction::update_line_number(int file, int sline, int lline, int hllfile, int hllsline)
{
  file_id = file;
  src_line = sline;
  lst_line = lline;
  hll_src_line = hllsline;
  hll_file_id = hllfile;
}

Invalid_Instruction::Invalid_Instruction(Processor *new_cpu,unsigned int new_opcode)
{
  cpu=new_cpu;
  opcode=new_opcode;
  sprintf(name_str,"INVALID");
}

char * Literal_op::name(char* return_str)
{
  sprintf(return_str,"%s\t0x%02x",name_str,L);
  
  return(return_str);
}

void Bit_op::decode(Processor *new_cpu, unsigned int new_opcode)
{
  opcode = new_opcode;

  cpu = new_cpu;
  
  mask = 1 << ((opcode >> 7) & 7);
  register_address = opcode & REG_MASK;
  access = 1;
  
  /*switch(cpu_pic->base_isa())
    {
    case _16BIT_PROCESSOR_:
      switch(cpu_pic->isa()) {
        case  _P18Cxx2_:
	case  _P18C2x2_:
	case  _P18C242_:
	case  _P18C252_:
	case  _P18C442_:
	case  _P18C452_:
	case  _P18F442_:
	case  _P18F248_:
	case  _P18F452_:
	case  _P18F1220_:
	case  _P18F1320_:
          mask = 1 << ((opcode >> 9) & 7);
          register_address = opcode & REG_MASK_16BIT;
          access = opcode & ACCESS_MASK_16BIT;
          if((!access) && (opcode & 0x80))
	    register_address |= 0xf00;

          break;
	case _P17C7xx_:
	case _P17C75x_:
	case _P17C756_:
	case _P17C756A_:
	case _P17C762_:
	case _P17C766_:
          mask = 1 << ((opcode >> 8) & 7);
          register_address = opcode & REG_MASK_16BIT;
          access = 0;
	  break;

	default:
          cout << "ERROR: (Bit_op) the processor is not defined\n";
	  break;
      }
      break;

    //case _14BIT_PROCESSOR_:
      mask = 1 << ((opcode >> 7) & 7);
      register_address = opcode & REG_MASK;
      access = 1;
      //break;

      case _12BIT_PROCESSOR_:
      mask = 1 << ((opcode >> 5) & 7);
      register_address = opcode & REG_MASK_12BIT;
      access = 1;
      break;
    default:
      cout << "ERROR: (Bit_op) the processor has a bad base type\n";
    }*/
}

char * Bit_op::name(char* return_str)
{
  //  %%% FIX ME %%% Actually just a slight dilemma - the source register will always be in
  //                 the lower bank of memory...

  reg = cpu->registers[register_address];

  unsigned int bit = 0;
  bit = ((opcode >> 7) & 7);
  
  /*switch(cpu_pic->base_isa())
    {
    case _16BIT_PROCESSOR_:
      bit = ((opcode >> 9) & 7);
      sprintf(return_str,"%s\t%s,%d,%c",
	      name_str,
	      reg->name(), 
	      bit,
	      access ? '1' : '0');

      return(return_str);
      break;

    case _14BIT_PROCESSOR_:
      bit = ((opcode >> 7) & 7);
      break;

    case _12BIT_PROCESSOR_:
      bit = ((opcode >> 5) & 7);
      break;
    default:
      bit = 0;
    }*/
  
  sprintf(return_str,"%s\t%s,%d",name_str,reg->name(), bit);
  //sprintf(return_str,"%s",name_str);
  
  return(return_str);
}


//----------------------------------------------------------------
//
// Register_op::name
//

char * Register_op::name(char* return_str)
{
  //  %%% FIX ME %%% Actually just a slight dilemma - the source register will always be in
  //                 the lower bank of memory (for the 12 and 14 bit cores).
  source = cpu->registers[register_address];

  //if(cpu_pic->base_isa() != _16BIT_PROCESSOR_)
    //sprintf(return_str,"%s\t%s,%c",name_str,source->name(), destination ? 'f' : 'w');
  /*else
    sprintf(return_str,"%s\t%s,%c,%c",
	    name_str,
	    source->name(), 
	    destination ? 'f' : 'w',
	    access ? '1' : '0');*/
  
  sprintf(return_str,"%s\t%s,%c",name_str,source->name(), destination ? 'f' : 'w');
  //sprintf(return_str,"%s",name_str);
  
  return(return_str);
}

//----------------------------------------------------------------
//
// Register_op::decode
//
// Base class to decode all 'register' type instructions. The main thing
// it does is obtains the register's address from the opcode. Note that this
// is processor dependent: in the 12-bit core processors, the register address
// is the lower 5 bits while in the 14-bit core it's the lower 7.

void  Register_op::decode(Processor *new_cpu, unsigned int new_opcode)
{ 
  opcode = new_opcode;
  cpu = new_cpu;
  
  register_address = opcode & REG_MASK;
  destination = opcode & DESTINATION_MASK;
  access = 1;

  /*switch(cpu_pic->base_isa())
    {
    case _16BIT_PROCESSOR_:
      destination = opcode & DESTINATION_MASK_16BIT;
      access = opcode & ACCESS_MASK_16BIT;
      register_address = opcode & REG_MASK_16BIT;
      if((!access) && (opcode & 0x80))
	register_address |= 0xf00;

      break;

    //case _14BIT_PROCESSOR_:
      register_address = opcode & REG_MASK;
      destination = opcode & DESTINATION_MASK;
      access = 1;
      //break;

      case _12BIT_PROCESSOR_:
      register_address = opcode & REG_MASK_12BIT;
      destination = opcode & DESTINATION_MASK_12BIT;
      access = 1;
      break;
_14BIT
    default:
      cout << "ERROR: (Register_op) the processor has a bad base type\n";
    }*/

}



// Instantiate an invalid instruction
Invalid_Instruction bad_instruction;

Register * Register_op::source = NULL;


//--------------------------------------------------

ADDLW::ADDLW (Processor *new_cpu, unsigned int new_opcode)
{
  decode(new_cpu, new_opcode);
  sprintf(name_str,"%s","ADDLW");
}

void ADDLW::execute(void)
{
  unsigned int old_value,new_value;

  ////trace.instruction(opcode);

  new_value = (old_value = cpu->W->value) + L;

  cpu->W->put_value(new_value & 0xff);
  cpu->status->put_Z_C_DC(new_value, old_value, L);

  cpu->pc->increment();

}



//--------------------------------------------------

RETFIE::RETFIE (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","RETFIE");

}

void RETFIE::execute(void)
{

  ////trace.instruction(opcode);

  cpu->pc->new_address(cpu->stack->pop());

  cpu->intcon->set_gie();
}

//--------------------------------------------------

RETURN::RETURN (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","RETURN");

}

void RETURN::execute(void)
{

  ////trace.instruction(opcode);

  cpu->pc->new_address(cpu->stack->pop());

}

//--------------------------------------------------

SUBLW::SUBLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","SUBLW");

}

void SUBLW::execute(void)
{
  unsigned int old_value,new_value;

//  //trace.instruction(opcode);

  new_value = L - (old_value = cpu->W->value);

  cpu->W->put_value(new_value & 0xff);

  cpu->status->put_Z_C_DC_for_sub(new_value, old_value, L);

  cpu->pc->increment();

}




//--------------------------------------------------

ADDWF::ADDWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);
  sprintf(name_str,"%s","ADDWF");

}

void ADDWF::execute(void)
{
  unsigned int new_value,src_value,w_value;

  ////trace.instruction(opcode);

  
  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];
  
  new_value = (src_value = source->get_value()) + (w_value = cpu->W->value);
  
  // Store the result

  if(destination)
    source->put_value(new_value & 0xff);      // Result goes to source
  else
    cpu->W->put_value(new_value & 0xff);

  
  cpu->status->put_Z_C_DC(new_value, src_value, w_value);
  cpu->pc->increment();

  

}

//--------------------------------------------------

ANDLW::ANDLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);
  sprintf(name_str,"%s","ANDLW");

}

void ANDLW::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  new_value = cpu->W->value & L;

  cpu->W->put_value(new_value);
  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

ANDWF::ANDWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);
  sprintf(name_str,"%s","ANDWF");

}

void ANDWF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = source->get_value() & cpu->W->value;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}


//--------------------------------------------------

BCF::BCF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  mask ^= 0xff;        // decode initializes the mask to 1<<bit

  sprintf(name_str,"%s","BCF");

}

void BCF::execute(void)
{

  //trace.instruction(opcode);

  if(!access)
    reg = cpu->registers[register_address];
  else
    reg = cpu->register_bank[register_address];

  //cout<<reg->name()<<endl;
  reg->put_value(reg->get_value() & mask);

  cpu->pc->increment();

}

//--------------------------------------------------

BSF::BSF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","BSF");

}

void BSF::execute(void)
{

  //trace.instruction(opcode);

  if(!access)
    reg = cpu->registers[register_address];
  else
    reg = cpu->register_bank[register_address];

  reg->put_value(reg->get_value() | mask);


  cpu->pc->increment();

}

//--------------------------------------------------

BTFSC::BTFSC (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","BTFSC");

}

void BTFSC::execute(void)
{

  //trace.instruction(opcode);

  if(!access)
    reg = cpu->registers[register_address];
  else
    reg = cpu->register_bank[register_address];

  cpu->pc->increment();

  if( 0 == (mask & reg->get_value()) )
    {
      cpu->pc->skip();                  // Skip next instruction
    }

}

//--------------------------------------------------

BTFSS::BTFSS (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","BTFSS");

}

void BTFSS::execute(void)
{

  //trace.instruction(opcode);

  if(!access)
    reg = cpu->registers[register_address];
  else
    reg = cpu->register_bank[register_address];

  cpu->pc->increment();

  if( 0 != (mask & reg->get_value()) )
    {
      cpu->pc->skip();                  // Skip next instruction
    }

}

//--------------------------------------------------
CALL::CALL (Processor *new_cpu, unsigned int new_opcode)
{
  opcode = new_opcode;
  cpu = new_cpu;
  destination = opcode&0x7ff;
  sprintf(name_str,"%s","CALL");
}

void CALL::execute(void)
{
  //trace.instruction(opcode);

  cpu->stack->push(cpu->pc->get_next());

  
  cpu->pc->jump(cpu->get_pclath_branching_jump() | destination);

}

char * CALL::name(char *return_str)
{

  sprintf(return_str,"%s\t0x%04x",name_str,destination);

  return(return_str);
}


//--------------------------------------------------
CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode)
{
  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","CLRF");
}

void CLRF::execute(void)
{
  //trace.instruction(opcode);

  if(!access)
    cpu->registers[register_address]->put_value(0);
  else
    cpu->register_bank[register_address]->put_value(0);

  cpu->status->put_Z(1);

  cpu->pc->increment();
}

char * CLRF::name(char *return_str)
{
  /*if(cpu == NULL)
     sprintf(return_str,"%s",name_str);
  else*/
     sprintf(return_str,"%s\t%s",name_str,cpu->registers[register_address]->name());

  return(return_str);
}


//--------------------------------------------------

CLRW::CLRW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","CLRW");

}

void CLRW::execute(void)
{

  //trace.instruction(opcode);
  cpu->W->put_value(0);

  cpu->status->put_Z(1);
  cpu->pc->increment();

}

//--------------------------------------------------

CLRWDT::CLRWDT (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","CLRWDT");

}

void CLRWDT::execute(void)
{

  //trace.instruction(opcode);
  //UPDATE when wdt done
  cpu->wdt.clear();

  
      cpu->status->put_TO(1);
      cpu->status->put_PD(1);
  

  cpu->pc->increment();

}

//--------------------------------------------------

COMF::COMF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","COMF");

}

void COMF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = source->get_value() ^ 0xff;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

DECF::DECF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","DECF");

}

void DECF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (source->get_value() - 1)&0xff;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

DECFSZ::DECFSZ (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","DECFSZ");

}

void DECFSZ::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (source->get_value() - 1)&0xff;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  if(0==new_value)
    cpu->pc->skip();                  // Skip next instruction

  cpu->pc->increment();

}

//--------------------------------------------------
GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode)
{
  opcode = new_opcode;
  cpu = new_cpu;

  destination = opcode&0x7ff;

  sprintf(name_str,"%s","GOTO");
}

void GOTO::execute(void)
{
  //trace.instruction(opcode);
  
  cpu->pc->jump(cpu->get_pclath_branching_jump() | destination);

}

char * GOTO::name(char *return_str)
{

  sprintf(return_str,"%s\t0x%04x",name_str,destination);

  return(return_str);
}


//--------------------------------------------------

INCF::INCF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","INCF");

}

void INCF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (source->get_value() + 1)&0xff;

  // Store the result

  if(destination)
    source->put_value(new_value);
  else
    cpu->W->put_value(new_value);


  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

INCFSZ::INCFSZ (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","INCFSZ");

}

void INCFSZ::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (source->get_value() + 1)&0xff;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  if(0==new_value)
    cpu->pc->skip();                  // Skip next instruction

  cpu->pc->increment();

}

//--------------------------------------------------

IORLW::IORLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","IORLW");

}

void IORLW::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  new_value = cpu->W->value | L;

  cpu->W->put_value(new_value);
  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

IORWF::IORWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","IORWF");

}

void IORWF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = source->get_value() | cpu->W->value;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","MOVLW");

}

void MOVLW::execute(void)
{
  

  //trace.instruction(opcode);

  cpu->W->put_value(L);

  cpu->pc->increment();

}

//--------------------------------------------------

MOVF::MOVF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","MOVF");

}

void MOVF::execute(void)
{
  unsigned int source_value;

  //trace.instruction(opcode);

  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  source_value = source->get_value();

  // Store the result

  if(destination)
    source->put_value(source_value);
  else
    cpu->W->put_value(source_value);


  cpu->status->put_Z(0==source_value);

  cpu->pc->increment();

}


//--------------------------------------------------
MOVWF::MOVWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","MOVWF");
}

void MOVWF::execute(void)
{
  //trace.instruction(opcode);

  if(!access)
    cpu->registers[register_address]->put_value(cpu->W->get_value());
  else
    cpu->register_bank[register_address]->put_value(cpu->W->get_value());

  cpu->pc->increment();
}

char * MOVWF::name(char *return_str)
{

 /*    sprintf(return_str,"%s",name_str);
     cout<<return_str<<endl;*/
     
  sprintf(return_str,"%s\t%s",name_str,cpu->registers[register_address]->name());

  return(return_str);
}


//--------------------------------------------------

NOP::NOP (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu,new_opcode);
  sprintf(name_str,"%s","NOP");

/*  // For the 18cxxx family, this 'nop' may in fact be the
  // 2nd word in a 2-word opcode. So just to be safe, let's
  // initialize the cross references to the source file.
  // (Subsequent initialization code will overwrite this,
  // but there is a chance that this info will be accessed
  // before that occurs).

  file_id = 0;
  src_line = 0;
  lst_line = 0;*/

}

void NOP::execute(void)
{

  //trace.instruction(opcode);

  cpu->pc->increment();

}

//--------------------------------------------------

OPTION::OPTION (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","OPTION");

}

void OPTION::execute(void)
{

  //trace.instruction(opcode);

  cpu->option_reg.put(cpu->W->get_value());

  cpu->pc->increment();

}

//--------------------------------------------------

RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","RETLW");

}

void RETLW::execute(void)
{

  //trace.instruction(opcode);

  cpu->W->put_value(L);
  cpu->pc->new_address(cpu->stack->pop());

}

//--------------------------------------------------

RLF::RLF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","RLF");

}

void RLF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (source->get_value() << 1) | cpu->status->get_C();

  if(destination)
    source->put_value(new_value&0xff);      // Result goes to source
  else
    cpu->W->put_value(new_value&0xff);

  cpu->status->put_C(new_value>0xff);

  cpu->pc->increment();

}

//--------------------------------------------------

RRF::RRF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","RRF");

}

void RRF::execute(void)
{
  unsigned int new_value,old_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  old_value = source->get_value();
  new_value = (old_value >> 1) | (cpu->status->get_C() ? 0x80 : 0);

  if(destination)
    source->put_value(new_value&0xff);      // Result goes to source
  else
    cpu->W->put_value(new_value&0xff);

  cpu->status->put_C(old_value&0x01);

  cpu->pc->increment();

}

//--------------------------------------------------

SLEEP::SLEEP (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","SLEEP");

}

void SLEEP::execute(void)
{
  // Implement WDT stuff
  //trace.instruction(opcode);

  cpu->status->put_TO(1);
  cpu->status->put_PD(0);

  bp.set_sleep();

}

//--------------------------------------------------

SUBWF::SUBWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","SUBWF");

}

void SUBWF::execute(void)
{
  unsigned int new_value,src_value,w_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = (src_value = source->get_value()) - (w_value = cpu->W->value);

  // Store the result

  if(destination)
    source->put_value(new_value & 0xff);      // Result goes to source
  else
    cpu->W->put_value(new_value & 0xff);

  cpu->status->put_Z_C_DC_for_sub(new_value, src_value, w_value);

  cpu->pc->increment();

}


//--------------------------------------------------

SWAPF::SWAPF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","SWAPF");

}

void SWAPF::execute(void)
{
  unsigned int src_value;

  //trace.instruction(opcode);


  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  src_value = source->get_value();

  if(destination)
    source->put_value( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
  else
    cpu->W->put_value( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );

  cpu->pc->increment();

}


//--------------------------------------------------
TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  // The TRIS instruction only uses the lower three bits to determine
  // the destination register

  register_address &= 7;

  // Furthermore, those three bits can only be 5,6, or 7

  if( (register_address > 7) || (register_address < 5))
    {
      //cout << "Warning: TRIS address '" << register_address << "' is  out of range\n";

	// set the address to a 'bad value' that's
	// easy to detect at run time:
	register_address = 0;
      
    }
  else {
  //   if(cpu->base_isa() == _14BIT_PROCESSOR_)
       register_address |= 0x80;  // The destination register is the TRIS
  }
  sprintf(name_str,"%s","tris");
}

void TRIS::execute(void)
{
  //trace.instruction(opcode);

  if(register_address)
    {
      // Execute the instruction only if the register is valid.
	cpu->registers[register_address]->put_value(cpu->W->get_value());
    }

  cpu->pc->increment();
}

char * TRIS::name(char *return_str)
{
  /*if(cpu == NULL)
     sprintf(return_str,"%s",name_str);
  else*/
     sprintf(return_str,"%s\t%s",name_str,cpu->registers[register_address]->name());

  return(return_str);
}


//--------------------------------------------------

XORLW::XORLW (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","XORLW");

}

void XORLW::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  new_value = cpu->W->value ^ L;

  cpu->W->put_value(new_value);
  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

//--------------------------------------------------

XORWF::XORWF (Processor *new_cpu, unsigned int new_opcode)
{

  decode(new_cpu, new_opcode);

  sprintf(name_str,"%s","XORWF");

}

void XORWF::execute(void)
{
  unsigned int new_value;

  //trace.instruction(opcode);

  if(!access)
    source = cpu->registers[register_address];
  else
    source = cpu->register_bank[register_address];

  new_value = source->get_value() ^ cpu->W->value;

  if(destination)
    source->put_value(new_value);      // Result goes to source
  else
    cpu->W->put_value(new_value);

  cpu->status->put_Z(0==new_value);

  cpu->pc->increment();

}

