#include "register.h"
#include <iostream>
#include "processor.h"
#include "interface.h"

using namespace std;

Register::Register()
{
   name_str1 = NULL;
   new_name("File Register");
}

Register::~Register()
{
   if(name_str1 != NULL)
      free(name_str1);
}

void Register::new_name(char* new_name)
{
   if(name_str1 != NULL)
      free(name_str1);

   if(new_name != NULL)
      name_str1 = strdup(new_name);
   else
      name_str1 = NULL;
}

int Register::getBit(unsigned int bit_number)
{
  return( (value >>  (bit_number & 0x07)) & 1 );
}
   
void Register::setBit(unsigned int bit_number, bool new_value)
{
   if(new_value==true)
   {
      unsigned int temp = 1<<bit_number;
      value = value | temp;
   }
   else
   {
      unsigned int temp = ~(1<<bit_number);
      value = value & temp;
   }
}

//------------------------------------------------------------------------
Invalid_File_Register::Invalid_File_Register(unsigned int at_address):File_Register()
{

  char name_str[100];
  //sprintf (name_str, "invalid fr 0x%02x", at_address);
  sprintf(name_str,"invalid");
  new_name(name_str);
}

FSR::FSR(void)
{
   new_name("FSR");
}

STATUS_Register::STATUS_Register(void)
{
  //break_point = 0;
  //break_on_z =0;
  //break_on_c =0;
  address = 3;
  rp_mask = RP_MASK;
  write_mask = 0xff & ~STATUS_TO & ~STATUS_PD;
  new_name("STATUS");
}

void inline STATUS_Register::put(unsigned int new_value)
{
   //value = new_value; // & STATUS_WRITABLE_BITS;

   value = (value & ~write_mask) | (new_value & write_mask);

   cpu->register_bank = &cpu->registers[(value & rp_mask) << 2]; 

   //trace.register_write(address,value);
}

void inline STATUS_Register::put_value(unsigned int new_value)
{ 
   put(new_value); 
}
//--------------------------------------------------
// WREG
//

unsigned int WREG::get(void)
{
  //trace.read_W(value);

  return(value);
}

void WREG::put(unsigned int new_value)
{

  value = new_value;
  //trace.write_W(value);

}

WREG::WREG(void)
{
  new_name("W");
}

//--------------------------------------------------
// member functions for the INDF class
//--------------------------------------------------
INDF::INDF(void)
{
  fsr_mask = 0x7f;           // for 14-bit core
  base_address_mask1 = 0;    //   "          "
  base_address_mask2 = 0xff; //   "          "
  new_name("INDF");
}


//this function has to be updated to actually put the new value to the FSRth register
void INDF::put(unsigned int new_value)
{

//  unsigned int reg = FSR_ADDR;
  /*trace.register_write(address,value);
  int reg = (cpu_pic->map_fsr_indf() + //cpu->fsr->value + 
	     ((cpu_pic->status->value & base_address_mask1)<<1) ) &  base_address_mask2;

  // if the fsr is 0x00 or 0x80, then it points to the indf
  if(reg & fsr_mask){
    cpu->registers[reg]->put(new_value);

    //(cpu->fsr->value & base_address_mask2) + ((cpu->status->value & base_address_mask1)<<1)
  }*/

}

/*void INDF::put_value(unsigned int new_value)
{

  // go ahead and use the regular put to write the data.
  // note that this is a 'virtual' function. Consequently,
  // all objects derived from a file_register should
  // automagically be correctly updated (which isn't
  // necessarily true if we just write new_value on top
  // of the current register value).

  put(new_value);

  if(xref)
    {
      xref->update();
      int r = (cpu_pic->map_fsr_indf() + //cpu->fsr->value + 
	       ((cpu_pic->status->value & base_address_mask1)<<1)& base_address_mask2);
      if(r & fsr_mask) 
	{
	  if(cpu->registers[r]->xref)
	    cpu->registers[r]->xref->update();
	}
    }

}*/

//this function has to be updated to actually get the value with the value of FSRth register
unsigned int INDF::get(void)
{

  /*trace.register_read(address,value);
  int reg = (cpu_pic->map_fsr_indf() + //cpu->fsr->value + 
	     ((cpu_pic->status->value & base_address_mask1)<<1) ) &  base_address_mask2;
  if(reg & fsr_mask)
    return(cpu->registers[reg]->get());
  else*/
    return(0);   // avoid infinite loop if fsr points to the indf
}

/*unsigned int INDF::get_value(void)
{
  int reg = (cpu_pic->fsr->value + 
	       ((cpu_pic->status->value & base_address_mask1)<<1) ) &  base_address_mask2;
  if(reg & fsr_mask)
    return(cpu->registers[reg]->get_value());
  else
    return(0);   // avoid infinite loop if fsr points to the indf
}*/


//--------------------------------------------------
// member functions for the OPTION base class
//--------------------------------------------------
OPTION_REG::OPTION_REG(void)
{
  por_value = 0xff;
  wdtr_value = 0xff;
  value = 0xff;
  new_name("OPTION");
}

void OPTION_REG::put_value(unsigned int new_value)
{
   OPTION_REG::put(new_value);
}

/*Must update to call functions to update tmr class*/
void OPTION_REG::put(unsigned int new_value)
{
  
  unsigned int old_value = value;
  value = new_value;

  // First, check the tmr0 clock source bit to see if we are  changing from
  // internal to external (or vice versa) clocks.
  if( (value ^ old_value) & T0CS)
      cpu->tmr0.new_clock_source();

  // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
  // increment if tmr0 is being clocked by an external clock?

  // Now check the rest of the tmr0 bits.
  


  if( (value ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
    cpu->tmr0.new_prescale();
  

  /*BY HRVOJE- This gives us seg fault, leaving out for now
  if( (value ^ old_value) & (PSA | PS2 | PS1 | PS0))
    cpu->wdt.new_prescale();*/

  /*if( (value ^ old_value) & (BIT6 | BIT7))
    cpu->option_new_bits_6_7(value & (BIT6 | BIT7));*/


  /*if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
    trace.register_write(address,value);
  else
    trace.write_OPTION(value);*/

}




//--------------------------------------------------
// member functions for the PCL base class
//--------------------------------------------------
PCL::PCL(void) : SFR_Register()
{
  new_name("PCL");
}

// %%% FIX ME %%% breaks are different

//CHANGE TO ALLOW JUMP OF PC
void PCL::put(unsigned int new_value)
{

  cpu->pc->computed_goto(new_value);
  //trace.register_write(address,value);
}

void PCL::put_value(unsigned int new_value)
{

  value = new_value;
  cpu->pc->put_value( (cpu->pc->get_value() & 0xffffff00) | value);

  // The gui (if present) will be updated in the pc->put_value call.
}

unsigned int PCL::get(void)
{
  return((value) & 0xff); //used to value + 1 but this may be incorrect
}

unsigned int PCL::get_value(void)
{
  return((value) & 0xff); //used to value + 1 but this may be incorrect
}





//--------------------------------------------------
// member functions for the PCLATH base class
//--------------------------------------------------

PCLATH::PCLATH(void)
{
  new_name("PCLATH");
}

void PCLATH::put(unsigned int new_value)
{
  value = new_value & PCLATH_MASK;

  //trace.register_write(address,value);
}

//UPDATE TO ACTUALLY PUT NEW VALUE TO PC
void PCLATH::put_value(unsigned int new_value)
{

  value = new_value & PCLATH_MASK;
  //cpu_pic->pc->put_value( (cpu_pic->pc->get_value() & 0xffff00ff) | (value<<8) );

  // The gui (if present) will be updated in the pc->put_value call.
}

unsigned int PCLATH::get(void)
{
  //trace.register_read(address,value);
  return(value & PCLATH_MASK);
}




//--------------------------------------------------
// member functions for the PCON base class
//--------------------------------------------------
//
PCON::PCON(void)
{
  valid_bits = BOR | POR;
}

void PCON::put(unsigned int new_value)
{

  value = (new_value&valid_bits);
  //trace.register_write(address,value);
}




//--------------------------------------------------
// member functions for the INTCON base class
//--------------------------------------------------
INTCON::INTCON(void)
{
  new_name("INTCON");
}

void INTCON::set_T0IF(void)
{

  value |= T0IF;

  //trace.register_write(address,value);

  /*if (value & (GIE | T0IE))
  {
    trace.interrupt();
  }*/
}

void INTCON::put(unsigned int new_value)
{

  value = new_value;
  //trace.register_write(address,value);

  // Now let's see if there's a pending interrupt
  // The INTCON bits are:
  // GIE | ---- | TOIE | INTE | RBIE | TOIF | INTF | RBIF
  // There are 3 sources for interrupts, TMR0, RB0/INTF
  // and RBIF (RB7:RB4 change). If the corresponding interrupt
  // flag is set AND the corresponding interrupt enable bit
  // is set AND global interrupts (GIE) are enabled, THEN
  // there's an interrupt pending.
  // note: bit6 is not handled here because it is processor
  // dependent (e.g. EEIE for x84 and ADIE for x7x).

  if(value & GIE)
    {

      if( (((value>>3)&value) & (T0IF | INTF | RBIF)) )
	{
	  //trace.interrupt();
	  bp.set_interrupt();
	}
      else if(value & XXIE)
	{
	  if(check_peripheral_interrupt())
	    {
	      //trace.interrupt();
	      bp.set_interrupt();
	    }
	}
    }
}

void INTCON::peripheral_interrupt(void)
{
  if(  (value & (GIE | XXIE)) == (GIE | XXIE) )
    bp.set_interrupt();
}




//-------------STACK METHODS-------------------------------------
Stack::Stack(void)
{

  stack_warnings_flag = 0;   // Do not display over/under flow stack warnings
  break_on_overflow = 0;     // Do not break if the stack over flows
  break_on_underflow = 0;    // Do not break if the stack under flows
  stack_mask = 7;            // 14 bit core.
  pointer = 0;

  for(int i=0; i<8; i++)
    contents[i] = 0;

}

//
// Stack::push
//
// push the passed address onto the stack by storing it at the current
// 


//NOTE: This may need to be changed to accomadate PIC stack functionality
void Stack::push(unsigned int address)
{

  // Write the address at the current point location. Note that the '& stack_mask'
  // implicitly handles the stack wrap around.
  contents[pointer & stack_mask] = address;

  // If the stack pointer is too big, then the stack has definitely over flowed.
  // However, some pic programs take advantage of this 'feature', so provide a means
  // for them to ignore the warnings.

  if(pointer++ >= stack_mask) {
    if(stack_warnings_flag || break_on_overflow)
    {  //cout << "stack overflow ";
    }
//BREAK ON OVERFLOW OF STACK
/*    if(break_on_overflow)
      bp.halt();*/
  }

}

//
// Stack::pop
//

unsigned int Stack::pop(void)
{

  // First decrement the stack pointer.

  if(--pointer < 0)  {
    if(stack_warnings_flag || break_on_underflow)
    {  //cout << "stack underflow ";
    }
/*IF BREAK ON UNDERFLOW
    if(break_on_underflow) 
      bp.halt();*/
  }


  
  return(contents[pointer & stack_mask]);
}

//
//  bool Stack::set_break_on_overflow(bool clear_or_set)
//
//  Set or clear the break on overflow flag


bool Stack::set_break_on_overflow(bool clear_or_set)
{
  if(break_on_overflow == clear_or_set)
    return 0;

  break_on_overflow = clear_or_set;

  return 1;

}

//
//  bool Stack::set_break_on_underflow(bool clear_or_set)
//
//  Set or clear the break on underflow flag


bool Stack::set_break_on_underflow(bool clear_or_set)
{
  if(break_on_underflow == clear_or_set)
    return 0;

  break_on_underflow = clear_or_set;

  return 1;

}



