//#include "processor.h"
#include "glibconfig.h"
#include "register.h"
#include "breakpoints.h"

#ifndef __TMR0_H__
#define __TMR0_H__

//---------------------------------------------------------
// TMR0 - Timer

//! \class
/*!
    \brief This class models Timer 0
*/
class TMR0 : public SFR_Register, public BreakCallBack
{
public:
  //! Prescale
  unsigned int prescale;
  //! Prescale counter
  unsigned int prescale_counter;
  //! Save option register contents here.
  unsigned int old_option;      
  //! Either on or off right now. 
  unsigned int state;    
  //! Synchronized cycle        
  guint64 synchronized_cycle;
  //! Future cycle
  guint64 future_cycle;
  //! Last cycle
  gint64 last_cycle;   

  //! Callback
  virtual void callback(void);

  //! Default constructor
  TMR0(void);

  //! Puts new value into TMR0
  /*!
      \param new_value New value
  */
  virtual void put(unsigned int new_value);
  //! Puts new value into TMR0
  /*!
      \param new_value New value
  */
  virtual void put_value(unsigned int new_value);
  unsigned int get(void);
  //! Gets value of TMR0
  /*!
      \return Value
  */
  virtual unsigned int get_value(void);
  //! Gets value of TMR0
  /*!
      \return Value
  */
  //! Starts the TMR0
  /*!
      \param new_value New Value
      \param sync Sync
  */
  virtual void start(int new_value,int sync=0);
  //! Stops TMR0
  virtual void stop(void);
  //! Used when tmr0 is attached to an external clock
  virtual void increment(void);   
  //! Used when tmr0 attached to internal oscillator
  virtual void internal_increment(void); 
  //! New prescale for TMR0
  virtual void new_prescale(void);
  //! Gets the prescale
  /*!
     \return Prescale
  */
  virtual unsigned int get_prescale(void);
  //! Gets the maximum counts
  /*!
      \return Maximum counts (256)
  */
  virtual unsigned int max_counts(void) {return 256;};
  //! New Clock Source
  void new_clock_source(void);
  //! Gets the T0CS
  /*!
      \return Value of T0CS
  */
  virtual unsigned int get_t0cs(void);
  //! Sets T0IF
  virtual void set_t0if(void);
  //! Resets TMR0
  /*!
      \param r Reset type
  */
  virtual void reset(RESET_TYPE r);
  //! Clears breakpoint from TMR0
  virtual void clear_break(void); 
  //! Prints callback
  virtual void callback_print(void);

};

#endif
