#ifndef WATCHREGISTER_H
#define WATCHREGISTER_H

#include "qstring.h"
#include "interface.h"

//! \class
/*!
    \brief This models a register that has been added to watch
    This class is only used for GUI purposes. It helps in adding
    a register chosen from the Register Viewer into the Watch Viewer.
*/
class WatchRegister {

   private:
      //! Pointer to register
      Register* reg;
      //! Register type
      REGISTER_TYPE typeVal;
      //! Breakpointer string
      QString bpStr;
      //! Type string
      QString typeStr;
      //! Name string
      QString nameStr;
      //! Address string
      QString addrStr;
      //! Decimal string
      QString decStr;
      //! Hex string
      QString hexStr;
      //! ASCII string
      QString asciiStr;
      //! Binary string
      QString binStr;
      
   public:
      //! Default constructor
      WatchRegister() {};
      
      //! Constructor
      /*!
          \param r Register
          \param t Register type
      */
      WatchRegister(Register* r,REGISTER_TYPE t) 
      {
          reg=r;
	      typeVal=t;
      };
	  
      //! Updates the attributes of this Watch Register
      void updateFields()
      {
          bpStr = "";
	      if(typeVal == REGISTER_RAM)
	          typeStr = "RAM";
	      else
	          typeStr = "EEPROM";
	      nameStr = reg->name();
	      addrStr.sprintf("0x%02x",reg->address);
	      decStr = QString::number(reg->get(),10);
	      hexStr.sprintf("0x%02x",reg->get());
	      asciiStr = char(reg->get());
          binStr = QString("%1").arg(reg->get(),8,2);
	      binStr.replace(' ',"0");
      };
      
      //! Returns the Breakpoint string
      /*!
          \return Breakpoint string
      */
      QString bp() {return bpStr;};
      //! Returns the type string
      /*!
          \return Type string
      */
      QString type() {return typeStr;};
      //! Returns the name string
      /*!
          \return Name string
      */
      QString name() {return nameStr;};
      //! Returns the address string
      /*!
          \return Address string
      */
      QString addr() {return addrStr;};
      //! Returns the decimal string
      /*!
          \return Decimal string
      */
      QString dec() {return decStr;};
      //! Returns the hex string
      /*!
          \return Hex string
      */
      QString hex() {return hexStr;};
      //! Returns the ASCII string
      /*!
          \return ASCII string
      */
      QString ascii() {return asciiStr;};
      //! Returns the binary string
      /*!
          \return Binary string
      */
      QString bin() {return binStr;};

};

#endif
