#include "wdt.h"
#include "processor.h"

//--------------------------------------------------
void WDT::update(void)
{
  
   //cout<<"WDTE: "<<wdte<<"\n";
  if(wdte){

    value = cpu->time_to_cycles(timeout);
    prescale = cpu->option_reg.get_psa() ? (cpu->option_reg.get_prescale()) : 0;

    if(future_cycle) {

      guint64 fc = cycles.value + value * (1<<prescale);

      //cout << "WDT::update:  moving break from " << future_cycle << " to " << fc << '\n';

      cycles.reassign_break(future_cycle, fc, this);
      future_cycle = fc;

    } else {
    
      //cout<<dec<<"After callback: Cycles.value = "<< cycles.value<<" WDT value= "<<value<<" prescale = "<<(1<<prescale)<<endl;
      future_cycle = cycles.value + value * (1<<prescale);

      cycles.set_break(future_cycle, this);
    }
  }
  //printf("Value after update = %d Prescale  = %d Future Cycle = %d\n",value,prescale,future_cycle);
  

}

//--------------------------------------------------
// WDT::put - shouldn't be called?
//

void WDT::put(unsigned int new_value)
{
  value = new_value;

  update();

}

void WDT::initialize(bool enable, double _timeout)
{
  break_point = 0;
  wdte = enable;
  timeout = _timeout;
  warned = 0;

  if(verbose)
    cout << " WDT init called "<< ( (enable) ? "enabling\n" :", but disabling WDT\n");

  if(wdte)
    {
      //cout << "Enabling WDT " << " timeout = " << timeout << " seconds\n";
      value = cpu->time_to_cycles(timeout);
      prescale = cpu->option_reg.get_psa() ? (cpu->option_reg.get_prescale()) : 0;

      //cout<<"Cycles.value = "<<cycles.value<<"WDT value = "<<value<<"Prescale factor = "<<(1<<prescale)<<endl;
      future_cycle = cycles.value + value * (1<<prescale);

      //cout <<"Return value of set_break should be 1 = "<<cycles.set_break(future_cycle, this)<<endl;;

    }

}

void WDT::callback(void)
{


  if(wdte) {
    //cout<<"WDT timeout: " << hex << cycles.value << '\n';

    future_cycle = 0;
    update();

    // The TO bit gets cleared when the WDT times out.
    cpu->status->put_TO(0);

    if(break_point)
      bp.halt();
    else if(bp.have_sleep()) {
      bp.clear_sleep();
    }else
      cpu->reset(WDT_RESET);
  }

}

void WDT::clear(void)
{
  if(wdte)
    update();
  else
    {
      if(!warned)
	{
	  warned = 1;
	  //cout << "The WDT is not enabled - clrwdt has no effect!\n";
	}
    }

}

void WDT::start_sleep(void)
{

  if(wdte) {
    prescale = 0;

    guint64 fc = cycles.value + value * (1<<prescale);

    //cout << "WDT::start_sleep:  moving break from " << future_cycle << " to " << fc << '\n';

    //cycles.reassign_break(future_cycle, fc, this);

    future_cycle = fc;
  }
}

void WDT::new_prescale(void)
{

  update();

}

void WDT::callback_print(void)
{

  cout << "WDT\n";
}
